@@ -1,8 +1,6 @@
 /* Dynamically allocated strings
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2003, 2004, 2005 Reuben Thomas.
+   Copyright (c) 2001, 2002, 2003, 2004, 2005, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -27,10 +25,10 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
+#include "xalloc.h"
 
 #include "astr.h"
-#include "zile.h"
-#include "extern.h"
+#include "xalloc_extra.h"
 
 #define ALLOCATION_CHUNK_SIZE	16
 
@@ -185,7 +183,7 @@ astr_substr (astr as, ptrdiff_t pos, size_t size)
 }
 
 int
-astr_cmp(astr as1, astr as2)
+astr_cmp (astr as1, astr as2)
 {
   return strcmp (as1->text, as2->text);
 }
@@ -309,12 +307,12 @@ assert_eq (astr as, const char *s)
 }
 
 /*
- * Stubs to make xmalloc_die happy.
+ * Stubs to make xalloc_die happy.
  */
 char *prog_name = "astr";
 
 void
-zile_exit (int doabort GCC_UNUSED)
+zile_exit (int doabort)
 {
   exit (2);
 }
