@@ -1,8 +1,6 @@
 /* Disk file handling
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008 Reuben Thomas.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -35,8 +33,9 @@
 #include <unistd.h>
 #include <utime.h>
 #include "dirname.h"
+#include "canonicalize.h"
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 int
@@ -83,134 +82,87 @@ agetcwd (void)
 /*
  * This functions does some corrections and expansions to
  * the passed path:
- * - expands `~/' and `~name/' expressions;
- * - replaces `//' with `/' (restarting from the root directory);
- * - removes `..' and `.' entries.
  *
- * If something goes wrong, the string is deleted and NULL returned.
+ * - expands `~/' and `~name/' expressions;
+ * - calls canonicalize_file_name on the result (a GNU function
+ *   that works like realpath but mallocs its result).
+ *
+ * The return value indicates success or failure.
  */
-astr
+bool
 expand_path (astr path)
 {
-  int ret = true;
+  int ok = true;
   struct passwd *pw;
-  const char *sp = astr_cstr (path);
+  const char *sp = astr_cstr (path), *p;
   astr epath = astr_new ();
 
-  if (*sp != '/')
+  for (p = sp; *p != '\0';)
     {
-      astr_cat_delete (epath, agetcwd ());
-      if (astr_len (epath) == 0 || *astr_char (epath, -1) != '/')
-        astr_cat_char (epath, '/');
-    }
+      if (*p == '~' && (p == sp || p[-1] == '/'))
+        {
+          /* Got `/~' or leading `~'.  Restart from this point. */
+          astr_truncate (epath, 0);
+          ++p;
 
-  while (*sp != '\0')
-    {
-      if (*sp == '/')
-        {
-          if (*++sp == '/')
+          if (*p == '/')
             {
-              /* Got `//'.  Restart from this point. */
-              while (*sp == '/')
-                sp++;
-              astr_truncate (epath, 0);
-            }
-          astr_cat_char (epath, '/');
-        }
-      else if (*sp == '~')
-        {
-          if (*(sp + 1) == '/')
-            {
-              /* Got `~/'. Restart from this point and insert the user's
-                 home directory. */
-              astr_truncate (epath, 0);
+              /* Got `~/'.  Insert the user's home directory. */
               pw = getpwuid (getuid ());
               if (pw == NULL)
                 {
-                  ret = false;
+                  ok = false;
                   break;
                 }
               if (strcmp (pw->pw_dir, "/") != 0)
                 astr_cat_cstr (epath, pw->pw_dir);
-              ++sp;
             }
           else
             {
-              /* Got `~something'.  Restart from this point and insert that
-                 user's home directory. */
+              /* Got `~something'.  Insert that user's home directory. */
               astr as = astr_new ();
-              astr_truncate (epath, 0);
-              ++sp;
-              while (*sp != '\0' && *sp != '/')
-                astr_cat_char (as, *sp++);
+              while (*p != '\0' && *p != '/')
+                astr_cat_char (as, *p++);
               pw = getpwnam (astr_cstr (as));
               astr_delete (as);
               if (pw == NULL)
                 {
-                  ret = false;
+                  ok = false;
                   break;
                 }
               astr_cat_cstr (epath, pw->pw_dir);
             }
         }
-      else if (*sp == '.')
+      else
+        while (*p != '\0' && *p != '/')
+          astr_cat_char (epath, *p++);
+
+      if (*p == '/')
+        astr_cat_char (epath, *p++);
+    }
+
+  if (ok)
+    {
+      char *s = canonicalize_file_name (astr_cstr (epath));
+      if (s != NULL)
         {
-          if (*(sp + 1) == '/' || *(sp + 1) == '\0')
-            {
-              ++sp;
-              if (*sp == '/' && *(sp + 1) != '/')
-                ++sp;
-            }
-          else if (*(sp + 1) == '.' &&
-                   (*(sp + 2) == '/' || *(sp + 2) == '\0'))
-            /* FIXME:
-             *
-             * If /foo on your system is a symlink to /bar/baz, then /foo/../quux
-             * is actually /bar/quux, not /quux as a naive ../-removal would give
-             * you. If you want to do this kind of processing, you probably want
-             * "Cwd"'s realpath function
-             */
-            {
-              if (astr_len (epath) >= 1 && *astr_char (epath, -1) == '/')
-                astr_truncate (epath, -1);
-              while (*astr_char (epath, -1) != '/' && astr_len (epath) >= 1)
-                astr_truncate (epath, -1);
-              sp += 2;
-              if (*sp == '/' && *(sp + 1) != '/')
-                ++sp;
-            }
-          else
-            goto got_component;
+          astr_cpy_cstr (epath, s);
+          free (s);
+
+          /* Restore trailing slash, removed by
+             canonicalize_file_name, if any. */
+          if (p[-1] == '/')
+            astr_cat_char (epath, '/');
+
+          astr_cpy (path, epath);
         }
       else
-        {
-          const char *p;
-        got_component:
-          p = sp;
-          while (*p != '\0' && *p != '/')
-            p++;
-          if (*p == '\0')
-            {			/* Final filename */
-              astr_cat_cstr (epath, sp);
-              break;
-            }
-          else
-            {			/* Non-final directory */
-              while (*sp != '/')
-                astr_cat_char (epath, *sp++);
-            }
-        }
+        ok = false;
     }
 
-  astr_cpy (path, epath);
   astr_delete (epath);
 
-  if (!ret)
-    {
-      astr_delete (path);
-      return NULL;
-    }
-  return path;
+  return ok;
 }
 
 /*
@@ -220,31 +172,25 @@ expand_path (astr path)
 astr
 compact_path (astr path)
 {
-  astr buf = astr_new ();
-  size_t i;
   struct passwd *pw = getpwuid (getuid ());
 
-  if (pw == NULL)
+  if (pw != NULL)
     {
-      /* User not found in password file. */
-      astr_cpy (buf, path);
-      return buf;
+      /* Replace `/userhome/' (if found) with `~/'. */
+      size_t i = strlen (pw->pw_dir);
+      if (!strncmp (pw->pw_dir, astr_cstr (path), i))
+        {
+          astr buf = astr_new_cstr ("~/");
+          if (!strcmp (pw->pw_dir, "/"))
+            astr_cat_cstr (buf, astr_char (path, 1));
+          else
+            astr_cat_cstr (buf, astr_char (path, i + 1));
+          astr_cpy (path, buf);
+          astr_delete (buf);
+        }
     }
 
-  /* Replace `/userhome/' (if existent) with `~/'. */
-  i = strlen (pw->pw_dir);
-  if (!strncmp (pw->pw_dir, astr_cstr (path), i))
-    {
-      astr_cpy_cstr (buf, "~/");
-      if (!strcmp (pw->pw_dir, "/"))
-        astr_cat_cstr (buf, astr_char (path, 1));
-      else
-        astr_cat_cstr (buf, astr_char (path, i + 1));
-    }
-  else
-    astr_cpy (buf, path);
-
-  return buf;
+  return path;
 }
 
 /*
@@ -391,7 +337,7 @@ read_from_disk (const char *filename)
   fclose (fp);
 }
 
-int
+bool
 find_file (const char *filename)
 {
   Buffer *bp;
@@ -453,17 +399,16 @@ creating one if none already exists.
 {
   astr buf = get_buffer_dir ();
   char *ms = minibuf_read_filename ("Find file: ", astr_cstr (buf), NULL);
-  int ret = false;
+
+  ok = leNIL;
 
   astr_delete (buf);
   if (ms == NULL)
-    return FUNCALL (keyboard_quit);
-
-  if (ms[0] != '\0')
-    ret = find_file (ms);
+    ok = FUNCALL (keyboard_quit);
+  else if (ms[0] != '\0')
+    ok = bool_to_lisp (find_file (ms));
 
   free (ms);
-  return bool_to_lisp (ret);
 }
 END_DEFUN
 
@@ -471,13 +416,12 @@ DEFUN ("find-file-read-only", find_file_read_only)
 /*+
 Edit the specified file but don't allow changes.
 Like `find-file' but marks buffer as read-only.
-Use M-x toggle-read-only to permit editing.
+Use @kbd{M-x toggle-read-only} to permit editing.
 +*/
 {
-  le * ret = FUNCALL (find_file);
-  if (ret == leT)
+  ok = FUNCALL (find_file);
+  if (ok == leT)
     cur_bp->flags |= BFLAG_READONLY;
-  return ret;
 }
 END_DEFUN
 
@@ -489,15 +433,12 @@ END_DEFUN
 static int
 check_modified_buffer (Buffer * bp)
 {
-  int ans;
-
   if (bp->flags & BFLAG_MODIFIED && !(bp->flags & BFLAG_NOSAVE))
     for (;;)
       {
-        if ((ans =
-             minibuf_read_yesno
-             ("Buffer %s modified; kill anyway? (yes or no) ",
-              bp->name)) == -1)
+        int ans = minibuf_read_yesno
+          ("Buffer %s modified; kill anyway? (yes or no) ", bp->name);
+        if (ans == -1)
           {
             FUNCALL (keyboard_quit);
             return false;
@@ -520,55 +461,58 @@ If the current buffer now contains an empty file that you just visited
   const char *buf = cur_bp->filename;
   char *base = base_name (cur_bp->filename);
   char *ms = minibuf_read_filename ("Find alternate: ", buf, base);
-  int ret = false;
+
+  ok = leNIL;
 
   if (ms == NULL)
-    return FUNCALL (keyboard_quit);
-
-  if (ms[0] != '\0' && check_modified_buffer (cur_bp))
+    ok = FUNCALL (keyboard_quit);
+  else if (ms[0] != '\0' && check_modified_buffer (cur_bp))
     {
       kill_buffer (cur_bp);
-      ret = find_file (ms);
+      ok = bool_to_lisp (find_file (ms));
     }
 
   free (ms);
   free (base);
-  return bool_to_lisp (ret);
 }
 END_DEFUN
 
-DEFUN ("switch-to-buffer", switch_to_buffer)
+DEFUN_ARGS ("switch-to-buffer", switch_to_buffer,
+            STR_ARG (buffer))
 /*+
 Select to the user specified buffer in the current window.
 +*/
 {
-  char *ms;
-  Buffer *swbuf;
-  Completion *cp;
+  Buffer *bp = ((cur_bp->next != NULL) ? cur_bp->next : head_bp);
 
-  swbuf = ((cur_bp->next != NULL) ? cur_bp->next : head_bp);
-
-  cp = make_buffer_completion ();
-  ms = minibuf_read_completion ("Switch to buffer (default %s): ",
-                                "", cp, NULL, swbuf->name);
-  free_completion (cp);
-  if (ms == NULL)
-    return FUNCALL (keyboard_quit);
-
-  if (ms[0] != '\0')
+  STR_INIT (buffer)
+  else
     {
-      swbuf = find_buffer (ms, false);
-      if (swbuf == NULL)
-        {
-          swbuf = find_buffer (ms, true);
-          swbuf->flags = BFLAG_NEEDNAME | BFLAG_NOSAVE;
-        }
+      Completion *cp = make_buffer_completion ();
+      buffer = minibuf_read_completion ("Switch to buffer (default %s): ",
+                                        "", cp, NULL, bp->name);
+      free_completion (cp);
+
+      if (buffer == NULL)
+        ok = FUNCALL (keyboard_quit);
     }
 
-  switch_to_buffer (swbuf);
-  free ((char *) ms);
+  if (ok == leT)
+    {
+      if (buffer && buffer[0] != '\0')
+        {
+          bp = find_buffer (buffer, false);
+          if (bp == NULL)
+            {
+              bp = find_buffer (buffer, true);
+              bp->flags = BFLAG_NEEDNAME | BFLAG_NOSAVE;
+            }
+        }
 
-  return leT;
+      switch_to_buffer (bp);
+    }
+
+  STR_FREE (buffer);
 }
 END_DEFUN
 
@@ -650,43 +594,48 @@ kill_buffer (Buffer * kill_bp)
     }
 }
 
-DEFUN ("kill-buffer", kill_buffer)
+DEFUN_ARGS ("kill-buffer", kill_buffer,
+            STR_ARG (buffer))
 /*+
-Kill the current buffer or the user specified one.
+Kill buffer BUFFER.
+With a nil argument, kill the current buffer.
 +*/
 {
   Buffer *bp;
-  char *ms;
-  Completion *cp;
 
-  cp = make_buffer_completion ();
-  ms = minibuf_read_completion ("Kill buffer (default %s): ",
-                                "", cp, NULL, cur_bp->name);
-  if (ms == NULL)
-    return FUNCALL (keyboard_quit);
-  free_completion (cp);
-  if (ms[0] != '\0')
+  STR_INIT (buffer)
+  else
     {
-      bp = find_buffer (ms, false);
+      Completion *cp = make_buffer_completion ();
+      buffer = minibuf_read_completion ("Kill buffer (default %s): ",
+                                        "", cp, NULL, cur_bp->name);
+      free_completion (cp);
+      if (buffer == NULL)
+        ok = FUNCALL (keyboard_quit);
+    }
+
+  if (buffer && buffer[0] != '\0')
+    {
+      bp = find_buffer (buffer, false);
       if (bp == NULL)
         {
-          minibuf_error ("Buffer `%s' not found", ms);
-          free ((char *) ms);
-          return leNIL;
+          minibuf_error ("Buffer `%s' not found", buffer);
+          free ((char *) buffer);
+          ok = leNIL;
         }
     }
   else
     bp = cur_bp;
 
-  if (check_modified_buffer (bp))
+  if (ok == leT)
     {
-      kill_buffer (bp);
-      free ((char *) ms);
-      return leT;
+      if (!check_modified_buffer (bp))
+        ok = leNIL;
+      else
+        kill_buffer (bp);
     }
 
-  free ((char *) ms);
-  return leNIL;
+  STR_FREE (buffer);
 }
 END_DEFUN
 
@@ -720,49 +669,56 @@ insert_buffer (Buffer * bp)
   undo_nosave = false;
 }
 
-DEFUN ("insert-buffer", insert_buffer)
+DEFUN_ARGS ("insert-buffer", insert_buffer,
+            STR_ARG (buffer))
 /*+
-Insert after point the contents of the user specified buffer.
+Insert after point the contents of BUFFER.
 Puts mark after the inserted text.
 +*/
 {
-  Buffer *bp, *swbuf;
-  char *ms;
+  Buffer *def_bp = ((cur_bp->next != NULL) ? cur_bp->next : head_bp);
   Completion *cp;
 
   if (warn_if_readonly_buffer ())
     return leNIL;
 
-  swbuf = ((cur_bp->next != NULL) ? cur_bp->next : head_bp);
-  cp = make_buffer_completion ();
-  ms = minibuf_read_completion ("Insert buffer (default %s): ",
-                                "", cp, NULL, swbuf->name);
-  if (ms == NULL)
-    return FUNCALL (keyboard_quit);
-  free_completion (cp);
-  if (ms[0] != '\0')
-    {
-      bp = find_buffer (ms, false);
-      if (bp == NULL)
-        {
-          minibuf_error ("Buffer `%s' not found", ms);
-          free ((char *) ms);
-          return leNIL;
-        }
-    }
+  STR_INIT (buffer)
   else
-    bp = swbuf;
+    {
+      cp = make_buffer_completion ();
+      buffer = minibuf_read_completion ("Insert buffer (default %s): ",
+                                        "", cp, NULL, def_bp->name);
+      if (buffer == NULL)
+        ok = FUNCALL (keyboard_quit);
+      free_completion (cp);
+    }
 
-  insert_buffer (bp);
-  set_mark_interactive ();
+  if (ok == leT)
+    {
+      Buffer *bp;
 
-  free ((char *) ms);
-  return leT;
+      if (buffer && buffer[0] != '\0')
+        {
+          bp = find_buffer (buffer, false);
+          if (bp == NULL)
+            {
+              minibuf_error ("Buffer `%s' not found", buffer);
+              ok = leNIL;
+            }
+        }
+      else
+        bp = def_bp;
+
+      insert_buffer (bp);
+      set_mark_interactive ();
+    }
+
+  STR_FREE (buffer);
 }
 END_DEFUN
 
 static int
-insert_file (char *filename)
+insert_file (const char *filename)
 {
   int fd;
   size_t size;
@@ -799,43 +755,31 @@ insert_file (char *filename)
   return true;
 }
 
-DEFUN ("insert-file", insert_file)
+DEFUN_ARGS ("insert-file", insert_file,
+       STR_ARG (file))
 /*+
-Insert contents of the user specified file into buffer after point.
+Insert contents of file FILENAME into buffer after point.
 Set mark after the inserted text.
 +*/
 {
-  char *ms;
-  astr buf;
-
   if (warn_if_readonly_buffer ())
     return leNIL;
 
-  buf = get_buffer_dir ();
-  ms = minibuf_read_filename ("Insert file: ", astr_cstr (buf), NULL);
-  if (ms == NULL)
+  STR_INIT (file)
+  else
     {
-      astr_delete (buf);
-      return FUNCALL (keyboard_quit);
-    }
-  astr_delete (buf);
-
-  if (ms[0] == '\0')
-    {
-      free (ms);
-      return leNIL;
+      astr buf = get_buffer_dir ();
+      file = minibuf_read_filename ("Insert file: ", astr_cstr (buf), NULL);
+      if (file == NULL)
+        ok = FUNCALL (keyboard_quit);
     }
 
-  if (!insert_file (ms))
-    {
-      free (ms);
-      return leNIL;
-    }
+  if (file == NULL || file[0] == '\0' || !insert_file (file))
+    ok = leNIL;
+  else
+    set_mark_interactive ();
 
-  set_mark_interactive ();
-
-  free (ms);
-  return leT;
+  STR_FREE (file);
 }
 END_DEFUN
 
@@ -929,15 +873,15 @@ copy_file (const char *source, const char *dest)
 }
 
 /*
- * Write buffer to given file name with given umask.
+ * Write buffer to given file name with given mode.
  */
 static int
-raw_write_to_disk (Buffer * bp, const char *filename, int umask)
+raw_write_to_disk (Buffer * bp, const char *filename, mode_t mode)
 {
   ssize_t eol_len = (ssize_t) strlen (bp->eol), written;
   Line *lp;
   int ret = 0;
-  int fd = open (filename, O_WRONLY | O_CREAT | O_TRUNC, umask);
+  int fd = creat (filename, mode);
 
   if (fd < 0)
     return -1;
@@ -995,8 +939,11 @@ create_backup_filename (const char *filename, const char *backupdir)
           ++filename;
         }
 
-      if (expand_path (buf) == NULL)
-        return NULL;
+      if (!expand_path (buf))
+        {
+          astr_delete (buf);
+          buf = NULL;
+        }
       res = buf;
     }
   else
@@ -1035,7 +982,8 @@ write_to_disk (Buffer * bp, char *filename)
       astr_delete (bfilename);
     }
 
-  ret = raw_write_to_disk (bp, filename, 0666);
+  ret = raw_write_to_disk (bp, filename, S_IRUSR | S_IWUSR |
+                           S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
   if (ret != 0)
     {
       if (ret == -1)
@@ -1103,7 +1051,7 @@ Save current buffer in visited file if modified. By default, makes the
 previous version into a backup file if this is the first save.
 +*/
 {
-  return bool_to_lisp (save_buffer (cur_bp));
+  ok = bool_to_lisp (save_buffer (cur_bp));
 }
 END_DEFUN
 
@@ -1115,7 +1063,7 @@ Makes buffer visit that file, and marks it not modified.
 {
   char *fname = cur_bp->filename != NULL ? cur_bp->filename : cur_bp->name;
   char *ms = minibuf_read_filename ("Write file: ", fname, NULL);
-
+  int ans = true;
 
   if (ms == NULL)
     return FUNCALL (keyboard_quit);
@@ -1125,22 +1073,31 @@ Makes buffer visit that file, and marks it not modified.
       return leNIL;
     }
 
-  //  if (file_exists (ms)
-      //(or (y-or-n-p (format "File `%s' exists; overwrite? " filename))
-// (error "Canceled")))
-
-  set_buffer_filename (cur_bp, ms);
-
-  cur_bp->flags &= ~(BFLAG_NEEDNAME | BFLAG_TEMPORARY);
-
-  if (write_to_disk (cur_bp, ms))
+  if (exist_file (ms))
     {
-      minibuf_write ("Wrote %s", ms);
-      cur_bp->flags &= ~BFLAG_MODIFIED;
+      ans = minibuf_read_yn ("File `%s' exists; overwrite? (y or n) ", ms);
+      if (ans == -1)
+        FUNCALL (keyboard_quit);
+      if (ans == false)
+        minibuf_error ("Canceled");
+      if (ans != true)
+        ok = leNIL;
+    }
+
+  if (ans == true)
+    {
+      set_buffer_filename (cur_bp, ms);
+      cur_bp->flags &= ~(BFLAG_NEEDNAME | BFLAG_TEMPORARY);
+      if (write_to_disk (cur_bp, ms))
+        {
+          minibuf_write ("Wrote %s", ms);
+          cur_bp->flags &= ~BFLAG_MODIFIED;
+        }
+      else
+        ok = leNIL;
     }
 
   free (ms);
-  return leT;
 }
 END_DEFUN
 
@@ -1226,7 +1183,7 @@ DEFUN ("save-some-buffers", save_some_buffers)
 Save some modified file-visiting buffers.  Asks user about each one.
 +*/
 {
-  return bool_to_lisp (save_some_buffers ());
+  ok = bool_to_lisp (save_some_buffers ());
 }
 END_DEFUN
 
@@ -1236,7 +1193,7 @@ Offer to save each buffer, then kill this Zile process.
 +*/
 {
   Buffer *bp;
-  int ans, i = 0;
+  int i = 0;
 
   if (!save_some_buffers ())
     return leNIL;
@@ -1248,18 +1205,16 @@ Offer to save each buffer, then kill this Zile process.
   if (i > 0)
     for (;;)
       {
-        if ((ans =
-             minibuf_read_yesno
-             ("Modified buffers exist; exit anyway? (yes or no) ", "")) == -1)
+        int ans = minibuf_read_yesno
+          ("Modified buffers exist; exit anyway? (yes or no) ");
+        if (ans == -1)
           return FUNCALL (keyboard_quit);
         else if (!ans)
           return leNIL;
         break;
       }
 
-  thisflag |= FLAG_QUIT_ZILE;
-
-  return leT;
+  thisflag |= FLAG_QUIT;
 }
 END_DEFUN
 
@@ -1286,7 +1241,7 @@ zile_exit (int doabort)
           astr_cpy_cstr (buf, bp->name);
         astr_cat_cstr (buf, ".ZILESAVE");
         fprintf (stderr, "Saving %s...\r\n", astr_cstr (buf));
-        raw_write_to_disk (bp, astr_cstr (buf), 0600);
+        raw_write_to_disk (bp, astr_cstr (buf), S_IRUSR | S_IWUSR);
         astr_delete (buf);
       }
 
