@@ -1,9 +1,6 @@
 /* Miscellaneous Emacs functions
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2003, 2004, 2005, 2006, 2007 Reuben Thomas.
-   Copyright (c) 2004 David A. Capello.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -33,7 +30,7 @@
 #include <errno.h>
 #include <unistd.h>
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 
@@ -43,7 +40,6 @@ Stop Zile and return to superior process.
 +*/
 {
   raise (SIGTSTP);
-  return leT;
 }
 END_DEFUN
 
@@ -54,7 +50,7 @@ Cancel current command.
 {
   deactivate_mark ();
   minibuf_error ("Quit");
-  return leNIL;
+  ok = leNIL;
 }
 END_DEFUN
 
@@ -75,7 +71,6 @@ With arg, turn Transient Mark mode on if arg is positive, off otherwise.
     set_variable ("transient-mark-mode", uniarg > 0 ? "t" : "nil");
 
   activate_mark ();
-  return leT;
 }
 END_DEFUN
 
@@ -219,12 +214,13 @@ Display a list of names of existing buffers.
 The list is displayed in a buffer named `*Buffer List*'.
 Note that buffers with names starting with spaces are omitted.
 
-The M column contains a * for buffers that are modified.
-The R column contains a % for buffers that are read-only.
+@itemize -
+The @samp{M} column contains a @samp{*} for buffers that are modified.
+The @samp{R} column contains a @samp{%} for buffers that are read-only.
+@end itemize
 +*/
 {
   write_temp_buffer ("*Buffer List*", true, write_buffers_list, cur_wp);
-  return leT;
 }
 END_DEFUN
 
@@ -233,12 +229,11 @@ DEFUN ("overwrite-mode", overwrite_mode)
 In overwrite mode, printing characters typed in replace existing
 text on a one-for-one basis, rather than pushing it to the right.
 At the end of a line, such characters extend the line.
-C-q still inserts characters in overwrite mode; this
+@kbd{C-q} still inserts characters in overwrite mode; this
 is supposed to make it easier to insert characters when necessary.
 +*/
 {
   cur_bp->flags ^= BFLAG_OVERWRITE;
-  return leT;
 }
 END_DEFUN
 
@@ -248,7 +243,6 @@ Change whether this buffer is visiting its file read-only.
 +*/
 {
   cur_bp->flags ^= BFLAG_READONLY;
-  return leT;
 }
 END_DEFUN
 
@@ -260,7 +254,6 @@ automatically breaks the line at a previous space.
 +*/
 {
   cur_bp->flags ^= BFLAG_AUTOFILL;
-  return leT;
 }
 END_DEFUN
 
@@ -271,35 +264,46 @@ Use C-u followed by a number to specify a column.
 Just C-u as argument means to use the current column.
 +*/
 {
-  size_t fill_col = (lastflag & FLAG_UNIARG_EMPTY) ? cur_bp->pt.o : (size_t) uniarg;
+  long fill_col = (lastflag & FLAG_UNIARG_EMPTY) ?
+    cur_bp->pt.o : (unsigned long) uniarg;
   char *buf;
-  le *branch;
 
   if (!(lastflag & FLAG_SET_UNIARG) && arglist == NULL)
     {
       fill_col = minibuf_read_number ("Set fill-column to (default %d): ", cur_bp->pt.o);
-      if (fill_col == SIZE_MAX)
+      if (fill_col == LONG_MAX)
         return leNIL;
-      else if (fill_col == SIZE_MAX - 1)
+      else if (fill_col == LONG_MAX - 1)
         fill_col = cur_bp->pt.o;
     }
 
-  if (arglist != NULL)
-    buf = arglist->next->data;
+  if (arglist)
+    {
+      if (arglist->next)
+        buf = arglist->next->data;
+      else
+        {
+          minibuf_error ("set-fill-column requires an explicit argument");
+          ok = leNIL;
+        }
+    }
   else
     {
-      xasprintf (&buf, "%d", fill_col);
+      xasprintf (&buf, "%ld", fill_col);
       /* Only print message when run interactively. */
       minibuf_write ("Fill column set to %s (was %d)", buf,
                      get_variable_number ("fill-column"));
     }
-  branch = leAddDataElement (leAddDataElement (leAddDataElement (NULL, "", 0), "fill-column", 0), buf, 0);
-  F_set_variable (0, branch);
+
+  if (ok == leT)
+    {
+      le *branch = leAddDataElement (leAddDataElement (leAddDataElement (NULL, "", 0), "fill-column", 0), buf, 0);
+      F_set_variable (0, branch);
+      leWipe (branch);
+    }
+
   if (arglist == NULL)
     free (buf);
-  leWipe (branch);
-
-  return leT;
 }
 END_DEFUN
 
@@ -317,7 +321,6 @@ Set mark at where point is.
 {
   set_mark_interactive ();
   activate_mark ();
-  return leT;
 }
 END_DEFUN
 
@@ -339,8 +342,6 @@ Put the mark where point is now, and point where the mark is now.
     activate_mark ();
 
   thisflag |= FLAG_NEED_RESYNC;
-
-  return leT;
 }
 END_DEFUN
 
@@ -352,7 +353,6 @@ Put point at beginning and mark at end of buffer.
   gotoeob ();
   FUNCALL (set_mark_command);
   gotobob ();
-  return leT;
 }
 END_DEFUN
 
@@ -403,8 +403,6 @@ You may also type up to 3 octal digits, to insert a character with that code.
     insert_char_in_insert_mode (c);
 
   minibuf_clear ();
-
-  return leT;
 }
 END_DEFUN
 
@@ -420,7 +418,7 @@ universal_argument (int keytype, int xarg)
   if (keytype == KBD_META)
     {
       astr_cpy_cstr (as, "ESC");
-      ungetkey ((size_t) (xarg + '0'));
+      pushkey ((size_t) (xarg + '0'));
     }
   else
     {
@@ -431,7 +429,7 @@ universal_argument (int keytype, int xarg)
   for (;;)
     {
       astr_cat_char (as, '-');	/* Add the '-' character. */
-      key = do_completion (as);
+      key = do_binding_completion (as);
       astr_truncate (as, -1);	/* Remove the '-' character. */
 
       /* Cancelled. */
@@ -492,14 +490,14 @@ universal_argument (int keytype, int xarg)
 DEFUN ("universal-argument", universal_argument)
 /*+
 Begin a numeric argument for the following command.
-Digits or minus sign following C-u make up the numeric argument.
-C-u following the digits or minus sign ends the argument.
-C-u without digits or minus sign provides 4 as argument.
-Repeating C-u without digits or minus sign multiplies the argument
+Digits or minus sign following @kbd{C-u} make up the numeric argument.
+@kbd{C-u} following the digits or minus sign ends the argument.
+@kbd{C-u} without digits or minus sign provides 4 as argument.
+Repeating @kbd{C-u} without digits or minus sign multiplies the argument
 by 4 each time.
 +*/
 {
-  return universal_argument (KBD_CTRL | 'u', 0);
+  ok = universal_argument (KBD_CTRL | 'u', 0);
 }
 END_DEFUN
 
@@ -667,17 +665,17 @@ when this can be done without changing the column they end at.
 The variable `tab-width' controls the spacing of tab stops.
 +*/
 {
-  return edit_tab_region (TAB_TABIFY);
+  ok = edit_tab_region (TAB_TABIFY);
 }
 END_DEFUN
 
 DEFUN ("untabify", untabify)
 /*+
 Convert all tabs in region to multiple spaces, preserving columns.
-The variable `tab-width' controls the spacing of tab stops.
+The variable @samp{tab-width} controls the spacing of tab stops.
 +*/
 {
-  return edit_tab_region (TAB_UNTABIFY);
+  ok = edit_tab_region (TAB_UNTABIFY);
 }
 END_DEFUN
 
@@ -693,7 +691,6 @@ Move point to the first non-whitespace character on this line.
         break;
       forward_char ();
     }
-  return leT;
 }
 END_DEFUN
 
@@ -745,7 +742,7 @@ Move point forward one word (backward if the argument is negative).
 With argument, do this that many times.
 +*/
 {
-  return execute_with_uniarg (false, uniarg, forward_word, backward_word);
+  ok = execute_with_uniarg (false, uniarg, forward_word, backward_word);
 }
 END_DEFUN
 
@@ -756,7 +753,7 @@ argument is negative).
 With argument, do this that many times.
 +*/
 {
-  return execute_with_uniarg (false, uniarg, backward_word, forward_word);
+  ok = execute_with_uniarg (false, uniarg, backward_word, forward_word);
 }
 END_DEFUN
 
@@ -880,7 +877,7 @@ With argument, do it that many times.  Negative arg -N means
 move backward across N balanced expressions.
 +*/
 {
-  return execute_with_uniarg (false, uniarg, forward_sexp, backward_sexp);
+  ok = execute_with_uniarg (false, uniarg, forward_sexp, backward_sexp);
 }
 END_DEFUN
 
@@ -891,7 +888,7 @@ With argument, do it that many times.  Negative arg -N means
 move forward across N balanced expressions.
 +*/
 {
-  return execute_with_uniarg (false, uniarg, backward_sexp, forward_sexp);
+  ok = execute_with_uniarg (false, uniarg, backward_sexp, forward_sexp);
 }
 END_DEFUN
 
@@ -1051,7 +1048,7 @@ and drag it forward past ARG other characters (backward if ARG negative).
 If no argument and at end of line, the previous two chars are exchanged.
 +*/
 {
-  return transpose (uniarg, forward_char, backward_char);
+  ok = transpose (uniarg, forward_char, backward_char);
 }
 END_DEFUN
 
@@ -1064,16 +1061,16 @@ If ARG is zero, the words around or after point and around or after mark
 are interchanged.
 +*/
 {
-  return transpose (uniarg, forward_word, backward_word);
+  ok = transpose (uniarg, forward_word, backward_word);
 }
 END_DEFUN
 
 DEFUN ("transpose-sexps", transpose_sexps)
 /*+
-Like M-x transpose-words but applies to sexps.
+Like @kbd{M-x transpose-words} but applies to sexps.
 +*/
 {
-  return transpose (uniarg, forward_sexp, backward_sexp);
+  ok = transpose (uniarg, forward_sexp, backward_sexp);
 }
 END_DEFUN
 
@@ -1084,7 +1081,7 @@ With argument ARG, takes previous line and moves it past ARG lines.
 With argument 0, interchanges line point is in with line mark is in.
 +*/
 {
-  return transpose (uniarg, next_line, previous_line);
+  ok = transpose (uniarg, next_line, previous_line);
 }
 END_DEFUN
 
@@ -1105,30 +1102,37 @@ DEFUN ("mark-word", mark_word)
 Set mark argument words away from point.
 +*/
 {
-  return mark (uniarg, F_forward_word);
+  ok = mark (uniarg, F_forward_word);
 }
 END_DEFUN
 
 DEFUN ("mark-sexp", mark_sexp)
 /*+
-Set mark argument sexps from point.
-The place mark goes is the same place C-M-f would
+Set mark @i{arg} sexps from point.
+The place mark goes is the same place @kbd{C-M-f} would
 move to with the same argument.
 +*/
 {
-  return mark (uniarg, F_forward_sexp);
+  ok = mark (uniarg, F_forward_sexp);
 }
 END_DEFUN
 
-DEFUN ("forward-line", forward_line)
+DEFUN_ARGS ("forward-line", forward_line,
+            INT_ARG (n))
 /*+
 Move N lines forward (backward if N is negative).
 Precisely, if point is on line I, move to the start of line I + N.
 +*/
 {
-  FUNCALL (beginning_of_line);
-
-  return execute_with_uniarg (false, uniarg, next_line, previous_line);
+  INT_INIT (n)
+  else
+    n = uniarg;
+  if (ok == leT)
+    {
+      FUNCALL (beginning_of_line);
+      ok = execute_with_uniarg (false, n, next_line, previous_line);
+    }
+  INT_FREE (n);
 }
 END_DEFUN
 
@@ -1163,7 +1167,7 @@ DEFUN ("backward-paragraph", backward_paragraph)
 Move backward to start of paragraph.  With argument N, do it N times.
 +*/
 {
-  return move_paragraph (uniarg, previous_line, next_line, F_beginning_of_line);
+  ok = move_paragraph (uniarg, previous_line, next_line, F_beginning_of_line);
 }
 END_DEFUN
 
@@ -1172,7 +1176,7 @@ DEFUN ("forward-paragraph", forward_paragraph)
 Move forward to end of paragraph.  With argument N, do it N times.
 +*/
 {
-  return move_paragraph (uniarg, next_line, previous_line, F_end_of_line);
+  ok = move_paragraph (uniarg, next_line, previous_line, F_end_of_line);
 }
 END_DEFUN
 
@@ -1194,8 +1198,6 @@ The paragraph marked is the one that contains point or follows point.
       FUNCALL (set_mark_command);
       FUNCALL_ARG (backward_paragraph, uniarg);
     }
-
-  return leT;
 }
 END_DEFUN
 
@@ -1240,8 +1242,6 @@ Fill paragraph at or after point.
   free_marker (m);
 
   undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-
-  return leT;
 }
 END_DEFUN
 
@@ -1309,7 +1309,7 @@ DEFUN ("downcase-word", downcase_word)
 Convert following word (or argument N words) to lower case, moving over.
 +*/
 {
-  return execute_with_uniarg (true, uniarg, setcase_word_lowercase, NULL);
+  ok = execute_with_uniarg (true, uniarg, setcase_word_lowercase, NULL);
 }
 END_DEFUN
 
@@ -1324,7 +1324,7 @@ DEFUN ("upcase-word", upcase_word)
 Convert following word (or argument N words) to upper case, moving over.
 +*/
 {
-  return execute_with_uniarg (true, uniarg, setcase_word_uppercase, NULL);
+  ok = execute_with_uniarg (true, uniarg, setcase_word_uppercase, NULL);
 }
 END_DEFUN
 
@@ -1341,7 +1341,7 @@ This gives the word(s) a first character in upper case and the rest
 lower case.
 +*/
 {
-  return execute_with_uniarg (true, uniarg, setcase_word_capitalize, NULL);
+  ok = execute_with_uniarg (true, uniarg, setcase_word_capitalize, NULL);
 }
 END_DEFUN
 
@@ -1393,7 +1393,7 @@ DEFUN ("upcase-region", upcase_region)
 Convert the region to upper case.
 +*/
 {
-  return setcase_region (UPPERCASE);
+  ok = setcase_region (UPPERCASE);
 }
 END_DEFUN
 
@@ -1402,7 +1402,7 @@ DEFUN ("downcase-region", downcase_region)
 Convert the region to lower case.
 +*/
 {
-  return setcase_region (LOWERCASE);
+  ok = setcase_region (LOWERCASE);
 }
 END_DEFUN
 
@@ -1415,7 +1415,7 @@ write_shell_output (va_list ap)
 }
 
 static bool
-pipe_command (const char *cmd, const char *tempfile, bool replace)
+pipe_command (const char *cmd, const char *tempfile, bool insert, bool replace)
 {
   astr out = astr_new (), s;
   size_t lines = 0;
@@ -1447,7 +1447,7 @@ pipe_command (const char *cmd, const char *tempfile, bool replace)
     minibuf_write ("(Shell command succeeded with no output)");
   else
     {
-      if (lastflag & FLAG_SET_UNIARG)
+      if (insert)
         {
           if (replace)
             {
@@ -1490,9 +1490,11 @@ minibuf_read_shell_command (void)
   return ms;
 }
 
-DEFUN ("shell-command", shell_command)
+DEFUN_ARGS ("shell-command", shell_command,
+            STR_ARG (cmd)
+            BOOL_ARG (insert))
 /*+
-Execute string command in inferior shell; display output, if any.
+Execute string @i{command} in inferior shell; display output, if any.
 With prefix argument, insert the command's output at point.
 
 Command is executed synchronously.  The output appears in the buffer
@@ -1500,22 +1502,29 @@ Command is executed synchronously.  The output appears in the buffer
 in the echo area, it is shown there, but it is nonetheless available
 in buffer `*Shell Command Output*' even though that buffer is not
 automatically displayed.
+
+The optional second argument @i{output-buffer}, if non-nil,
+says to insert the output in the current buffer.
 +*/
 {
-  bool ret;
-  char *cmd = minibuf_read_shell_command ();
+  STR_INIT (cmd)
+  else
+    cmd = minibuf_read_shell_command ();
+  BOOL_INIT (insert)
+  else
+    insert = lastflag & FLAG_SET_UNIARG;
 
-  if (cmd == NULL)
-    return leNIL;
+  if (cmd != NULL)
+    ok = bool_to_lisp (pipe_command (cmd, "/dev/null", insert, false));
 
-  ret = pipe_command (cmd, "/dev/null", false);
-  free (cmd);
-
-  return bool_to_lisp (ret);
+  STR_FREE (cmd);
+  BOOL_FREE (insert);
 }
 END_DEFUN
 
-DEFUN ("shell-command-on-region", shell_command_on_region)
+DEFUN_ARGS ("shell-command-on-region", shell_command_on_region,
+            STR_ARG (cmd)
+            BOOL_ARG (insert))
 /*+
 Execute string command in inferior shell with region as input.
 Normally display output (if any) in temp buffer `*Shell Command Output*';
@@ -1528,46 +1537,56 @@ there.  Otherwise it is displayed in the buffer `*Shell Command Output*'.
 The output is available in that buffer in both cases.
 +*/
 {
-  int fd;
-  ssize_t written;
-  char tempfile[] = P_tmpdir "/zileXXXXXX";
-  Region r;
-  bool ret;
-  char *cmd = minibuf_read_shell_command (), *p;
+  STR_INIT (cmd)
+  else
+    cmd = minibuf_read_shell_command ();
+  BOOL_INIT (insert)
+  else
+    insert = lastflag & FLAG_SET_UNIARG;
 
-  if (cmd == NULL)
-    return leNIL;
-
-  if (!calculate_the_region (&r))
-    return leNIL;
-
-  fd = mkstemp (tempfile);
-  if (fd == -1)
+  if (cmd != NULL)
     {
-      minibuf_error ("Cannot open temporary file");
-      return leNIL;
-    }
+      Region r;
 
-  p = copy_text_block (r.start.n, r.start.o, r.size);
-  written = write (fd, p, r.size);
-  free (p);
-  close (fd);
-
-  if (written != (ssize_t) r.size)
-    {
-      if (written == -1)
-        minibuf_error ("Error writing to temporary file: %s",
-                       strerror (errno));
+      if (!calculate_the_region (&r))
+        ok = leNIL;
       else
-        minibuf_error ("Error writing to temporary file");
-      return leNIL;
+        {
+          char tempfile[] = P_tmpdir "/zileXXXXXX";
+          int fd = mkstemp (tempfile);
+
+          if (fd == -1)
+            {
+              minibuf_error ("Cannot open temporary file");
+              ok = leNIL;
+            }
+          else
+            {
+              char *p = copy_text_block (r.start.n, r.start.o, r.size);
+              ssize_t written = write (fd, p, r.size);
+
+              free (p);
+              close (fd);
+
+              if (written != (ssize_t) r.size)
+                {
+                  if (written == -1)
+                    minibuf_error ("Error writing to temporary file: %s",
+                                   strerror (errno));
+                  else
+                    minibuf_error ("Error writing to temporary file");
+                  ok = leNIL;
+                }
+            }
+
+          if (ok == leT)
+            ok = bool_to_lisp (pipe_command (cmd, tempfile, insert, true));
+
+          remove (tempfile);
+        }
     }
 
-  ret = pipe_command (cmd, tempfile, true);
-  free (cmd);
-  remove (tempfile);
-
-  return bool_to_lisp (ret);
+  STR_FREE (cmd);
 }
 END_DEFUN
 
@@ -1599,11 +1618,11 @@ Delete the text between point and mark.
   Region r;
 
   if (!calculate_the_region (&r))
-    return leNIL;
-  if (!delete_region (&r))
-    return leNIL;
-  deactivate_mark ();
-  return leT;
+    ok = leNIL;
+  else if (!delete_region (&r))
+    ok = leNIL;
+  else
+    deactivate_mark ();
 }
 END_DEFUN
 
@@ -1690,7 +1709,5 @@ On nonblank line, delete any immediately following blank lines.
 
   free_marker (old_marker);
   deactivate_mark ();
-
-  return leT;
 }
 END_DEFUN
