@@ -1,8 +1,6 @@
 /* Lisp parser
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 2001 Scott "Jerry" Lawrence.
-   Copyright (c) 2005 Reuben Thomas.
+   Copyright (c) 2001, 2005, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -24,7 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 void
@@ -196,24 +194,41 @@ lisp_read (le * list, astr as, ptrdiff_t * pos)
     }
 }
 
-bool
-lisp_load (const char *file)
+void
+lisp_loadstring (astr as)
 {
-  FILE *fp;
-  le *list;
+  ptrdiff_t pos = 0;
+  le * list = lisp_read (NULL, as, &pos);
+
+  leEval (list);
+  leWipe (list);
+}
+
+bool
+lisp_loadfile (const char *file)
+{
+  FILE *fp = fopen (file, "r");
 
-  fp = fopen (file, "r");
   if (fp != NULL)
     {
       astr bs = astr_fread (fp);
-      ptrdiff_t pos = 0;
-      list = lisp_read (NULL, bs, &pos);
-      leEval (list);
+      lisp_loadstring (bs);
       astr_delete (bs);
-      leWipe (list);
       fclose (fp);
       return true;
     }
 
     return false;
 }
+
+DEFUN ("load", load)
+/*+
+Execute a file of Lisp code named FILE.
++*/
+{
+  if (arglist && countNodes (arglist) >= 2)
+    ok = bool_to_lisp (lisp_loadfile (arglist->next->data));
+  else
+    ok = leNIL;
+}
+END_DEFUN
