@@ -1,8 +1,6 @@
 /* Program invocation, startup and shutdown
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2004, 2005, 2006, 2007, 2008 Reuben Thomas.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -36,11 +34,11 @@
 #include "dirname.h"
 #include "gl_linked_list.h"
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 #define ZILE_COPYRIGHT_STRING \
-  "Copyright (C) 2008 Free Software Foundation, Inc."
+  "Copyright (C) 2009 Free Software Foundation, Inc."
 
 /* The executable name */
 char *prog_name = PACKAGE;
@@ -70,9 +68,9 @@ loop (void)
       thisflag = lastflag & FLAG_DEFINING_MACRO;
       key = getkey ();
       minibuf_clear ();
-      process_key (key);
+      process_key (root_bindings, key);
 
-      if (thisflag & FLAG_QUIT_ZILE)
+      if (thisflag & FLAG_QUIT)
         break;
       if (!(thisflag & FLAG_SET_UNIARG))
         last_uniarg = 1;
@@ -90,21 +88,17 @@ static char about_splash_str[] = "\
 " ZILE_COPYRIGHT_STRING "\n\
 \n\
 Type `C-x C-c' to exit " PACKAGE_NAME ".\n\
-Type `C-h h' for help; `C-x u; to undo changes.\n\
-Type `C-h C-d' for information on getting the latest version.\n\
-Type `C-h t' for a tutorial on using " PACKAGE_NAME ".\n\
-Type `C-h s' for a sample configuration file.\n\
+Type `C-x u; to undo changes.\n\
 Type `C-g' at any time to quit the current operation.\n\
 \n\
 `C-x' means hold the CTRL key while typing the character `x'.\n\
 `M-x' means hold the META or ALT key down while typing `x'.\n\
 If there is no META or ALT key, instead press and release\n\
 the ESC key and then type `x'.\n\
-Combinations like `C-h h' mean first press `C-h', then `h'.\n\
+Combinations like `C-x u' mean first press `C-x', then `u'.\n\
 ";
 
-static char about_minibuf_str[] =
-  "Welcome to " PACKAGE_NAME "!  For help type `C-h h'";
+static char about_minibuf_str[] = "Welcome to " PACKAGE_NAME "!";
 
 static void
 about_screen (void)
@@ -119,16 +113,21 @@ about_screen (void)
 }
 
 static void
-execute_functions (gl_list_t funcs)
+load_files (gl_list_t files)
 {
   size_t i;
-  for (i = 0; i < gl_list_size (funcs); i++)
+  for (i = 0; i < gl_list_size (files); i++)
     {
-      char *func = (char *) gl_list_get_at (funcs, i);
+      char *file = (char *) gl_list_get_at (files, i);
       term_redisplay ();
-      if (!execute_function (func, 1))
-        minibuf_error ("Function `%s' not defined", func);
       lastflag |= FLAG_NEED_RESYNC;
+      if (!lisp_loadfile (file))
+        {
+          minibuf_error ("Cannot open load file: %s\n", file);
+          break;
+        }
+      if (thisflag & FLAG_QUIT)
+        break;
     }
 }
 
@@ -212,12 +211,15 @@ signal_init (void)
 }
 
 /* Options table */
-/* Options which take no argument have optional_argument, so that no
-   arguments are signalled as extraneous, to mimic Emacs */
 struct option longopts[] = {
-  {"help", optional_argument, NULL, 'h'},
-  {"no-init-file", optional_argument, NULL, 'q'},
-  {"version", optional_argument, NULL, 'v'},
+#define D(text)
+#define O(longname, shortname, arg, argstring, docstring) \
+  {longname, arg, NULL, shortname},
+#define A(argstring, docstring)
+#include "tbl_opts.h"
+#undef D
+#undef O
+#undef A
   {0, 0, 0, 0}
 };
 
@@ -225,7 +227,7 @@ int
 main (int argc, char **argv)
 {
   int c, qflag = false;
-  gl_list_t fargs = gl_list_create_empty (GL_LINKED_LIST,
+  gl_list_t l_args = gl_list_create_empty (GL_LINKED_LIST,
                                           NULL, NULL, NULL, false);
   size_t line;
 
@@ -244,9 +246,10 @@ main (int argc, char **argv)
   for (;;)
     {
       int this_optind = optind ? optind : 1;
+      char *buf;
 
       /* Leading : so as to return ':' for a missing arg, not '?' */
-      c = getopt_long_only (argc, argv, ":h:qv", longopts, NULL);
+      c = getopt_long (argc, argv, ":l:q", longopts, NULL);
 
       if (c == -1)
         break;
@@ -257,37 +260,42 @@ main (int argc, char **argv)
           qflag = true;
           break;
         case 'v':
-          fprintf (stdout,
-                   ZILE_VERSION_STRING "\n"
-                   ZILE_COPYRIGHT_STRING "\n"
-                   "GNU " PACKAGE_NAME " comes with ABSOLUTELY NO WARRANTY.\n"
-                   "You may redistribute copies of " PACKAGE_NAME "\n"
-                   "under the terms of the GNU General Public License.\n"
-                   "For more information about these matters, see the file named COPYING.\n");
+          printf (ZILE_VERSION_STRING "\n"
+                  ZILE_COPYRIGHT_STRING "\n"
+                  "GNU " PACKAGE_NAME " comes with ABSOLUTELY NO WARRANTY.\n"
+                  "You may redistribute copies of " PACKAGE_NAME "\n"
+                  "under the terms of the GNU General Public License.\n"
+                  "For more information about these matters, see the file named COPYING.\n");
           return 0;
         case 'h':
-          fprintf (stdout,
-                   "Usage: %s [OPTION-OR-FILENAME]...\n"
-                   "\n"
-                   "Run " PACKAGE_NAME ", the lightweight Emacs clone.\n"
-                   "\n"
-                   "Initialization options:\n"
-                   "\n"
-                   "--help                  display this help message and exit\n"
-                   "--no-init-file, -q      do not load ~/." PACKAGE "\n"
-                   "--version               display version information and exit\n"
-                   "\n" "Action options:\n" "\n"
-                   "FILE                    visit FILE using find-file\n"
-                   "+LINE FILE              visit FILE using find-file, then go to line LINE\n"
-                   "\n"
-                   "Report bugs to " PACKAGE_BUGREPORT ".\n",
-                   prog_name);
+          printf ("Usage: %s [OPTION-OR-FILENAME]...\n"
+                  "\n"
+                  "Run " PACKAGE_NAME ", the lightweight Emacs clone.\n"
+                  "\n",
+                  argv[0]);
+#define D(text) \
+          printf (text "\n");
+#define O(longname, shortname, arg, argstring, docstring) \
+          xasprintf (&buf, "--%s, -%c %s", longname, shortname, argstring); \
+          printf ("%-24s%s\n", buf, docstring);                          \
+          free (buf);
+#define A(argstring, docstring) \
+          printf ("%-24s%s\n", argstring, docstring);
+#include "tbl_opts.h"
+#undef D
+#undef O
+#undef A
+          printf ("\n"
+                  "Report bugs to " PACKAGE_BUGREPORT ".\n");
           return 0;
+        case 'l':
+          gl_list_add_last (l_args, (void *) optarg);
+          break;
         case '?':		/* Unknown option */
           minibuf_error ("Unknown option `%s'", argv[this_optind]);
           break;
         case ':':		/* Missing argument */
-          fprintf (stderr, "%s: Option `%s' requires an argument\n\n",
+          fprintf (stderr, "%s: Option `%s' requires an argument\n",
                    prog_name, argv[this_optind]);
           exit (1);
         }
@@ -300,7 +308,7 @@ main (int argc, char **argv)
 
   setlocale (LC_ALL, "");
 
-  init_bindings ();
+  init_default_bindings ();
   init_minibuf ();
 
   term_init ();
@@ -317,7 +325,7 @@ main (int argc, char **argv)
       if (as)
         {
           astr_cat_cstr (as, "/." PACKAGE);
-          lisp_load (astr_cstr (as));
+          lisp_loadfile (astr_cstr (as));
           astr_delete (as);
         }
     }
@@ -341,7 +349,7 @@ main (int argc, char **argv)
 
   /* Show the splash screen only if no files and no Lisp expression
      or load file is specified on the command line. */
-  if (minibuf_contents == NULL && argc == 0 && gl_list_size (fargs) == 0)
+  if (minibuf_contents == NULL && argc == 0 && gl_list_size (l_args) == 0)
     about_screen ();
 
   setup_main_screen (argc);
@@ -356,17 +364,17 @@ main (int argc, char **argv)
       free (buf);
     }
 
-  execute_functions (fargs);
-  gl_list_free (fargs);
+  load_files (l_args);
+  gl_list_free (l_args);
 
   /* Run the main loop. */
-  loop ();
+  if (!(thisflag & FLAG_QUIT))
+    loop ();
 
   /* Tidy and close the terminal. */
-  term_tidy ();
-  term_close ();
+  term_finish ();
 
-  free_bindings ();
+  free_bindings (root_bindings);
   free_eval ();
 
   /* Free Lisp state. */
