@@ -1,7 +1,6 @@
 /* Minibuffer facility functions
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -28,7 +27,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 static History *files_history = NULL;
@@ -118,12 +117,12 @@ minibuf_read (const char *fmt, const char *value, ...)
 /*
  * Read a non-negative number from the minibuffer.
  */
-size_t
+unsigned long
 minibuf_read_number (const char *fmt, ...)
 {
   va_list ap;
   char *buf;
-  size_t n;
+  unsigned long n;
 
   va_start (ap, fmt);
   xvasprintf (&buf, fmt, ap);
@@ -139,17 +138,14 @@ minibuf_read_number (const char *fmt, ...)
           break;
         }
       if (strlen (ms) == 0)
-        n = SIZE_MAX - 1;
+        n = ULONG_MAX - 1;
       else
-        {
-          unsigned long un = strtoul (ms, NULL, 10);
-          n = un == ULONG_MAX ? SIZE_MAX : (size_t) un;
-        }
+        n = strtoul (ms, NULL, 10);
       free (ms);
-      if (n == SIZE_MAX)
+      if (n == ULONG_MAX)
         minibuf_write ("Please enter a number.");
     }
-  while (n == SIZE_MAX);
+  while (n == ULONG_MAX);
 
   return n;
 }
@@ -171,8 +167,8 @@ minibuf_read_filename (const char *fmt, const char *value,
   xvasprintf (&buf, fmt, ap);
   va_end (ap);
 
-  as = expand_path (astr_new_cstr (value));
-  if (as)
+  as = astr_new_cstr (value);
+  if (expand_path (as))
     {
       as = compact_path (as);
 
@@ -182,22 +178,22 @@ minibuf_read_filename (const char *fmt, const char *value,
         pos -= strlen (file);
       p = term_minibuf_read (buf, astr_cstr (as), pos, cp, files_history);
       free_completion (cp);
-      astr_delete (as);
       free (buf);
 
       if (p != NULL)
         {
-          as = expand_path (astr_new_cstr (p));
-          if (as)
+          astr as = astr_new_cstr (p);
+          if (expand_path (as))
             {
               add_history_element (files_history, p);
               p = xstrdup (astr_cstr (as));
-              astr_delete (as);
             }
           else
             p = NULL;
+          astr_delete (as);
         }
     }
+  astr_delete (as);
 
   return p;
 }
@@ -208,6 +204,35 @@ minibuf_test_in_completions (const char *ms, gl_list_t completions)
   return gl_sortedlist_search (completions, completion_strcmp, ms) != NULL;
 }
 
+int
+minibuf_read_yn (const char *fmt, ...)
+{
+  va_list ap;
+  char *buf, *errmsg = "";
+
+  va_start (ap, fmt);
+  xvasprintf (&buf, fmt, ap);
+  va_end (ap);
+
+  for (;;) {
+    size_t key;
+
+    minibuf_write ("%s%s", errmsg, buf);
+    key = getkey ();
+    switch (key)
+      {
+      case 'y':
+        return true;
+      case 'n':
+        return false;
+      case KBD_CTRL | 'g':
+        return -1;
+      default:
+        errmsg = "Please answer y or n.  ";
+      }
+  }
+}
+
 int
 minibuf_read_yesno (const char *fmt, ...)
 {
@@ -215,25 +240,27 @@ minibuf_read_yesno (const char *fmt, ...)
   char *ms;
   const char *errmsg = "Please answer yes or no.";
   Completion *cp = completion_new (false);
-  int retvalue = false;
+  int ret = -1;
 
   gl_sortedlist_add (cp->completions, completion_strcmp, xstrdup ("yes"));
   gl_sortedlist_add (cp->completions, completion_strcmp, xstrdup ("no"));
 
   va_start (ap, fmt);
-  ms = vminibuf_read_completion (fmt, "", cp, NULL, errmsg,
+  ms = minibuf_vread_completion (fmt, "", cp, NULL, errmsg,
                                  minibuf_test_in_completions, errmsg, ap);
   va_end (ap);
 
   if (ms != NULL)
     {
-      gl_list_node_t n = gl_sortedlist_search (cp->completions, completion_strcmp, ms);
+      gl_list_node_t n = gl_sortedlist_search (cp->completions,
+                                               completion_strcmp, ms);
       assert (n);
-      retvalue = !strcmp ((char *) gl_list_node_value (cp->completions, n), "yes");
+      ret = !strcmp ((char *) gl_list_node_value (cp->completions, n),
+                     "yes");
     }
   free_completion (cp);
 
-  return retvalue;
+  return ret;
 }
 
 char *
@@ -257,7 +284,7 @@ minibuf_read_completion (const char *fmt, char *value, Completion * cp,
  * Read a string from the minibuffer using a completion.
  */
 char *
-vminibuf_read_completion (const char *fmt, char *value, Completion * cp,
+minibuf_vread_completion (const char *fmt, char *value, Completion * cp,
                           History * hp, const char *empty_err,
                           bool (*test) (const char *s, gl_list_t completions),
                           const char *invalid_err, va_list ap)
