@@ -1,8 +1,6 @@
 /* Minibuffer handling
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2003, 2004, 2005 Reuben Thomas.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -28,7 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 void
@@ -131,6 +129,7 @@ do_minibuf_read (const char *prompt, const char *value, size_t pos,
           term_clrtoeol ();
           if (saved)
             astr_delete (saved);
+          astr_delete (as);
           return NULL;
         case KBD_CTRL | 'a':
         case KBD_HOME:
@@ -155,7 +154,7 @@ do_minibuf_read (const char *prompt, const char *value, size_t pos,
             ding ();
           break;
         case KBD_CTRL | 'k':
-          /* FIXME: no kill-register save is done yet. */
+          /* FIXME: do kill-register save. */
           if (pos < astr_len (as))
             astr_truncate (as, pos);
           else
