@@ -1,8 +1,6 @@
 /* Redisplay engine
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
-   Copyright (c) 2003, 2004, 2005, 2006, 2007 Reuben Thomas.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -28,7 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "zile.h"
+#include "main.h"
 #include "config.h"
 #include "extern.h"
 
@@ -424,6 +422,16 @@ term_tidy (void)
   term_refresh ();
 }
 
+/*
+ * Tidy and close the terminal ready to leave Zile.
+ */
+void
+term_finish (void)
+{
+  term_tidy ();
+  term_close ();
+}
+
 /*
  * Add a string to the terminal
  */
